"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var AutoUpdateConfig_1 = require("./AutoUpdateConfig");
var AutoUpdateService_1 = require("./AutoUpdateService");
var Evaluator_1 = require("./Evaluator");
var Const = require("./Consts");
var pckg = require('../package.json');
var FloodGateClient = /** @class */ (function (_super) {
    __extends(FloodGateClient, _super);
    function FloodGateClient(_config) {
        var _this = _super.call(this) || this;
        _this.isReady = false;
        var self = _this;
        _this.config = _config;
        _this.Version = pckg.version;
        _this.config.logger.Log("SDK Version = " + _this.Version);
        if (_config && _config instanceof AutoUpdateConfig_1.AutoUpdateConfig) {
            _this.service = new AutoUpdateService_1.AutoUpdateService(_config);
            _this.config.logger.Log("Setting service to AutoUpdateService");
        }
        // elseif additional services...
        else {
            throw new Error("Invalid service");
        }
        _this.service.on('ready', function () {
            self.isReady = true;
            self.emit(Const.EVENT_SDK_READY);
            self.config.logger.Log("FloodGateClient() Ready");
        }).Start();
        return _this;
    }
    FloodGateClient.prototype.IsReady = function () {
        return this.isReady;
    };
    /**
     * Return a value of the requested flag or the default value
     * Fetch data from remote server if not available in cache
     * @param {string} _key - Flag key
     * @param {string} _defaultValue - Default value to return to the client if no flag is found
     * @param callback - Client callback method
     * @param _user - User to evaluate against
     */
    FloodGateClient.prototype.FetchValue = function (_key, _defaultValue, callback, _user) {
        this.user = _user;
        try {
            this.service.GetFlags(function (value) {
                if (value) {
                    var evaluator = new Evaluator_1.Evaluator();
                    var result = evaluator.Evaluate(_key, value, _defaultValue, _user);
                    // Return flag value to caller
                    callback(result);
                }
                else {
                    callback(_defaultValue);
                }
            });
        }
        catch (error) {
            callback(_defaultValue);
        }
    };
    /**
     * Return a value of the requested flag or the default value.
     *
     * This method does not use callbacks, if the value is not present in the cache then the
     * default value is returned or if the client is not ready an error is thrown.
     * @param {string} _key - Flag key
     * @param {string} _defaultValue - Default value to return to the client if no flag is found
     * @param _user - User to evaluate against
     */
    FloodGateClient.prototype.GetValue = function (_key, _defaultValue, _user) {
        if (!this.isReady) {
            throw "Client is not ready! Try using FetchValue()";
        }
        this.user = _user;
        try {
            var flags = this.service.GetFlagsLocal();
            if (flags) {
                var evaluator = new Evaluator_1.Evaluator();
                var result = evaluator.Evaluate(_key, flags, _defaultValue, _user);
                return result;
            }
            return _defaultValue;
        }
        catch (error) {
            return _defaultValue;
        }
    };
    return FloodGateClient;
}(events_1.EventEmitter));
exports.FloodGateClient = FloodGateClient;
