"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var Cache_1 = require("./Cache");
var ApiService_1 = require("./ApiService");
var Consts = require("./Consts");
var ServiceBase = /** @class */ (function (_super) {
    __extends(ServiceBase, _super);
    function ServiceBase(_config) {
        var _this = _super.call(this) || this;
        _this.config = _config;
        _this.cache = new Cache_1.InMemoryCache();
        if (_config.cache) {
            _this.cache = _config.cache;
        }
        return _this;
    }
    /**
     * Fetch environment data from the server
     * Check if the data has been modified and if not return from cache
     * @param callback
     */
    ServiceBase.prototype.FetchRemote = function (callback) {
        var _this = this;
        var requestHeaders = [
            {
                key: "Accept",
                value: "application/json"
            },
            {
                key: "Content-Type",
                value: "application/json"
            },
            {
                key: "FloodGate-SDK-Agent",
                value: "js-v" + Consts.SDK_VERSION
            }
        ];
        var api = new ApiService_1.ApiService().setHeaders(requestHeaders);
        var url = this.config.buildUrl();
        var etagCacheKey = this.config.sdkKey + "_" + Consts.CACHE_ETAG;
        // Check to see if we have a current ETag
        var etag = this.cache.Get(etagCacheKey);
        if (etag) {
            api.setHeaders([{ key: "If-None-Match", value: "" + etag }]);
        }
        // Check to see if we have a last modified
        // let lastModified: string | null = this.cache.Get("Last-Modified");
        api.get(url)
            .then(function (response) {
            var responseHeaders = response.headers;
            etag = responseHeaders.get("ETag");
            if (typeof (etag) === "string") {
                etag = etag.replace(/['"]+/g, '');
            }
            // Check current etag vs cached etag, return cached data if matched
            if ((etag !== null && etag !== undefined)) {
                if (etag == _this.cache.Get(etagCacheKey)) {
                    var json_1 = _this.cache.Get(_this.config.sdkKey);
                    callback(json_1);
                    return;
                }
                _this.cache.Set(etagCacheKey, etag);
            }
            var json = response.parsedBody;
            if (_this.validateJson(json)) {
                _this.cache.Set(_this.config.sdkKey, json);
                callback(json);
            }
        })
            .catch(function (error) {
            // Return cache if available
            var json = _this.cache.Get(_this.config.sdkKey);
            if (json) {
                callback(json);
            }
            callback(null);
            // throw error;
        });
    };
    ServiceBase.prototype.FetchLocal = function () {
        // const etagCacheKey = `${this.config.sdkKey}_${Consts.CACHE_ETAG}`;
        var json = this.cache.Get(this.config.sdkKey);
        if (json) {
            return json;
        }
        return;
    };
    ServiceBase.prototype.validateJson = function (json) {
        return true;
    };
    return ServiceBase;
}(events_1.EventEmitter));
exports.ServiceBase = ServiceBase;
